@@ -17,6 +17,7 @@ import (
 	"path/filepath"
 	"strconv"
 	"strings"
+	"time"
 )
 
 const (
@@ -195,7 +196,9 @@ func confLoadBinaryKey(out *bool, rec *IniRecord, vFalse, vTrue string) error {
 func confLoadLogLevelKey(out *LogLevel, rec *IniRecord) error {
 	var mask LogLevel
 	for _, s := range strings.Split(rec.Value, ",") {
+		s = strings.TrimSpace(s)
 		switch s {
+		case "":
 		case "error":
 			mask |= LogError
 		case "info":
@@ -208,8 +211,10 @@ func confLoadLogLevelKey(out *LogLevel, rec *IniRecord) error {
 			mask |= LogTraceESCL | LogDebug | LogInfo | LogError
 		case "trace-http":
 			mask |= LogTraceHTTP | LogDebug | LogInfo | LogError
+		case "trace-usb":
+			mask |= LogTraceUSB | LogDebug | LogInfo | LogError
 		case "all", "trace-all":
-			mask |= LogAll
+			mask |= LogAll & ^LogTraceUSB
 		default:
 			return confBadValue(rec, "invalid log level %q", s)
 		}
@@ -219,6 +224,33 @@ func confLoadLogLevelKey(out *LogLevel, rec *IniRecord) error {
 	return nil
 }
 
+// Load QuirksResetMethod key
+func confLoadQuirksResetMethodKey(out *QuirksResetMethod, rec *IniRecord) error {
+	switch rec.Value {
+	case "none":
+		*out = QuirksResetNone
+		return nil
+	case "soft":
+		*out = QuirksResetSoft
+		return nil
+	case "hard":
+		*out = QuirksResetHard
+		return nil
+	default:
+		return confBadValue(rec, "must be none, soft or hard")
+	}
+}
+
+// Load time.Duration key
+func confLoadDurationKey(out *time.Duration, rec *IniRecord) error {
+	var ms uint
+	err := confLoadUintKey(&ms, rec)
+	if err == nil {
+		*out = time.Millisecond * time.Duration(ms)
+	}
+	return err
+}
+
 // Load size key
 func confLoadSizeKey(out *int64, rec *IniRecord) error {
 	units := uint64(1)
