@@ -26,7 +26,13 @@ const (
 )
 
 // pnpRetryTime returns time of next retry of failed device initialization
-func pnpRetryTime() time.Time {
+func pnpRetryTime(err error) time.Time {
+	if err == ErrBlackListed || err == ErrUnusable {
+		// These errors are unrecoverable.
+		// Forget about device for the next million hours :-)
+		return time.Now().Add(time.Hour * 1e6)
+	}
+
 	return time.Now().Add(DevInitRetryInterval)
 }
 
@@ -52,6 +58,12 @@ func PnPStart(exitWhenIdle bool) PnPExitReason {
 		os.Signal(syscall.SIGTERM),
 		os.Signal(syscall.SIGHUP))
 
+	// Start control socket server
+	err := CtrlsockStart()
+	if err == nil {
+		defer CtrlsockStop()
+	}
+
 	// Serve PnP events until terminated
 loop:
 	for {
@@ -70,11 +82,13 @@ loop:
 			for _, addr := range added {
 				Log.Debug('+', "PNP %s: added", addr)
 				dev, err := NewDevice(dev_descs[addr])
+				StatusSet(addr, dev_descs[addr], err)
+
 				if err == nil {
 					devByAddr[addr] = dev
 				} else {
 					Log.Error('!', "PNP %s: %s", addr, err)
-					retryByAddr[addr] = pnpRetryTime()
+					retryByAddr[addr] = pnpRetryTime(err)
 				}
 			}
 
@@ -82,6 +96,7 @@ loop:
 			for _, addr := range removed {
 				Log.Debug('-', "PNP %s: removed", addr)
 				delete(retryByAddr, addr)
+				StatusDel(addr)
 
 				dev, ok := devByAddr[addr]
 				if ok {
@@ -98,12 +113,14 @@ loop:
 
 				Log.Debug('+', "PNP %s: retry", addr)
 				dev, err := NewDevice(dev_descs[addr])
+				StatusSet(addr, dev_descs[addr], err)
+
 				if err == nil {
 					devByAddr[addr] = dev
 					delete(retryByAddr, addr)
 				} else {
 					Log.Error('!', "PNP %s: %s", addr, err)
-					retryByAddr[addr] = pnpRetryTime()
+					retryByAddr[addr] = pnpRetryTime(err)
 				}
 			}
 		}
@@ -124,6 +141,7 @@ loop:
 			tickerRunning = true
 		}
 
+		// Wait for the next event
 		select {
 		case <-UsbHotPlugChan:
 		case <-ticker.C:
