@@ -18,6 +18,7 @@ import (
 	"path/filepath"
 	"sort"
 	"strings"
+	"time"
 )
 
 // Quirks represents device-specific quirks
@@ -27,17 +28,56 @@ type Quirks struct {
 	Blacklist        bool              // Blacklist the device
 	HttpHeaders      map[string]string // HTTP header override
 	UsbMaxInterfaces uint              // Max number of USB interfaces
+	DisableFax       bool              // Disable fax for device
+	ResetMethod      QuirksResetMethod // Device reset method
+	InitDelay        time.Duration     // Delay before 1st IPP-USB request
+	RequestDelay     time.Duration     // Delay between IPP-USB requests
 	Index            int               // Incremented in order of loading
 }
 
+// QuirksResetMethod represents how to reset a device
+// during initialization
+type QuirksResetMethod int
+
+// QuirksResetUnset - reset method not specified
+// QuirksResetNone  - don't reset device at all
+// QuirksResetSoft  - use class-specific soft reset
+// QuirksResetHard  - use USB hard reset
+const (
+	QuirksResetUnset QuirksResetMethod = iota
+	QuirksResetNone
+	QuirksResetSoft
+	QuirksResetHard
+)
+
+// String returns textual representation of QuirksResetMethod
+func (m QuirksResetMethod) String() string {
+	switch m {
+	case QuirksResetUnset:
+		return "unset"
+	case QuirksResetNone:
+		return "none"
+	case QuirksResetSoft:
+		return "soft"
+	case QuirksResetHard:
+		return "hard"
+	}
+
+	return fmt.Sprintf("unknown (%d)", int(m))
+}
+
 // empty returns true, if Quirks are actually empty
 func (q *Quirks) empty() bool {
 	return !q.Blacklist &&
 		len(q.HttpHeaders) == 0 &&
-		q.UsbMaxInterfaces == 0
+		q.UsbMaxInterfaces == 0 &&
+		!q.DisableFax &&
+		q.ResetMethod == QuirksResetUnset &&
+		q.InitDelay == 0 &&
+		q.RequestDelay == 0
 }
 
-// QuirksSet represents collection of quirks, indexed by model name
+// QuirksSet represents collection of quirks
 type QuirksSet []*Quirks
 
 // LoadQuirksSet creates new QuirksSet and loads its content from a directory
@@ -104,7 +144,7 @@ func (qset *QuirksSet) readFile(file string) error {
 				HttpHeaders: make(map[string]string),
 				Index:       len(*qset),
 			}
-			*qset = append(*qset, q)
+			qset.Add(q)
 
 			continue
 		} else if q == nil {
@@ -128,6 +168,19 @@ func (qset *QuirksSet) readFile(file string) error {
 		case "usb-max-interfaces":
 			err = confLoadUintKeyRange(&q.UsbMaxInterfaces, rec,
 				1, math.MaxUint32)
+
+		case "disable-fax":
+			err = confLoadBinaryKey(&q.DisableFax, rec,
+				"false", "true")
+
+		case "init-reset":
+			err = confLoadQuirksResetMethodKey(&q.ResetMethod, rec)
+
+		case "init-delay":
+			err = confLoadDurationKey(&q.InitDelay, rec)
+
+		case "request-delay":
+			err = confLoadDurationKey(&q.RequestDelay, rec)
 		}
 	}
 
@@ -138,7 +191,13 @@ func (qset *QuirksSet) readFile(file string) error {
 	return err
 }
 
-// Get quirks by model name
+// Add appends Quirks to QuirksSet
+func (qset *QuirksSet) Add(q *Quirks) {
+	*qset = append(*qset, q)
+}
+
+// ByModelName returns a subset of quirks, applicable for
+// specific device, matched by model name
 //
 // In a case of multiple match, quirks are returned in
 // the from most prioritized to least prioritized order
@@ -147,7 +206,7 @@ func (qset *QuirksSet) readFile(file string) error {
 // more prioritized entry, it is removed from the less
 // prioritized entries. Entries, that in result become
 // empty, are removed at all
-func (qset QuirksSet) Get(model string) []Quirks {
+func (qset QuirksSet) ByModelName(model string) QuirksSet {
 	type item struct {
 		q        *Quirks
 		matchlen int
@@ -171,37 +230,33 @@ func (qset QuirksSet) Get(model string) []Quirks {
 	})
 
 	// Rebuild it into the slice of *Quirks
-	quirks := make([]Quirks, len(list))
+	quirks := make(QuirksSet, len(list))
 	for i := range list {
-		quirks[i] = *list[i].q
-	}
-
-	// If at least one Quirks contains Blacklist == true,
-	// it overrides everything else.
-	//
-	// Note, we check it after building and sorting the entire
-	// list for more accurate logging
-	for _, q := range quirks {
-		if q.Blacklist {
-			return []Quirks{q}
-		}
+		quirks[i] = list[i].q
 	}
 
 	// Remove duplicates and empty entries
 	httpHeaderSeen := make(map[string]struct{})
 	out := 0
 	for in, q := range quirks {
-		q.HttpHeaders = make(map[string]string)
+		// Note, here we avoid modification of the HttpHeaders
+		// map in the original Quirks structure
+		//
+		// Unfortunately, Golang misses immutable types,
+		// so we must be very careful here
+		q2 := &Quirks{}
+		*q2 = *q
+		q2.HttpHeaders = make(map[string]string)
 
 		for name, value := range quirks[in].HttpHeaders {
 			if _, seen := httpHeaderSeen[name]; !seen {
 				httpHeaderSeen[name] = struct{}{}
-				q.HttpHeaders[name] = value
+				q2.HttpHeaders[name] = value
 			}
 		}
 
-		if !q.empty() {
-			quirks[out] = q
+		if !q2.empty() {
+			quirks[out] = q2
 			out++
 		}
 	}
@@ -210,3 +265,72 @@ func (qset QuirksSet) Get(model string) []Quirks {
 
 	return quirks
 }
+
+// GetBlacklist returns effective Blacklist parameter,
+// taking the whole set into consideration
+func (qset QuirksSet) GetBlacklist() bool {
+	for _, q := range qset {
+		if q.Blacklist {
+			return true
+		}
+	}
+
+	return false
+}
+
+// GetUsbMaxInterfaces returns effective UsbMaxInterfaces parameter,
+// taking the whole set into consideration
+func (qset QuirksSet) GetUsbMaxInterfaces() uint {
+	for _, q := range qset {
+		if q.UsbMaxInterfaces != 0 {
+			return q.UsbMaxInterfaces
+		}
+	}
+
+	return 0
+}
+
+// GetDisableFax returns effective DisableFax parameter,
+// taking the whole set into consideration
+func (qset QuirksSet) GetDisableFax() bool {
+	for _, q := range qset {
+		if q.DisableFax {
+			return true
+		}
+	}
+
+	return false
+}
+
+// GetResetMethod returns effective ResetMethod parameter
+func (qset QuirksSet) GetResetMethod() QuirksResetMethod {
+	for _, q := range qset {
+		if q.ResetMethod != QuirksResetUnset {
+			return q.ResetMethod
+		}
+	}
+
+	return QuirksResetNone
+}
+
+// GetInitDelay returns effective InitDelay parameter
+func (qset QuirksSet) GetInitDelay() time.Duration {
+	for _, q := range qset {
+		if q.InitDelay != 0 {
+			return q.InitDelay
+		}
+	}
+
+	return 0
+}
+
+// GetRequestDelay returns effective RequestDelay parameter
+func (qset QuirksSet) GetRequestDelay() time.Duration {
+	for _, q := range qset {
+		if q.RequestDelay != 0 {
+			return q.RequestDelay
+		}
+	}
+
+	return 0
+}
