@@ -35,7 +35,7 @@ type UsbTransport struct {
 	connReleased chan struct{} // Signalled when connection released
 	shutdown     chan struct{} // Closed by Shutdown()
 	connstate    *usbConnState // Connections state tracker
-	quirks       []Quirks      // Device quirks
+	quirks       QuirksSet     // Device quirks
 	deadline     time.Time     // Deadline for requests
 }
 
@@ -68,7 +68,7 @@ func NewUsbTransport(desc UsbDeviceDesc) (*UsbTransport, error) {
 	transport.log.SetLevels(Conf.LogDevice)
 
 	// Setup quirks
-	transport.quirks = Conf.Quirks.Get(transport.info.MfgAndProduct)
+	transport.quirks = Conf.Quirks.ByModelName(transport.info.MfgAndProduct)
 
 	// Write device info to the log
 	log := transport.log.Begin().
@@ -89,6 +89,12 @@ func NewUsbTransport(desc UsbDeviceDesc) (*UsbTransport, error) {
 		log.Debug(' ', "  from [%s] (%s)", quirks.Model, quirks.Origin)
 		log.Debug(' ', "    blacklist = %v", quirks.Blacklist)
 		log.Debug(' ', "    usb-max-interfaces = %v", quirks.UsbMaxInterfaces)
+		log.Debug(' ', "    disable-fax = %v", quirks.DisableFax)
+		log.Debug(' ', "    init-delay = %s", quirks.InitDelay)
+		log.Debug(' ', "    request-delay = %s", quirks.RequestDelay)
+		if quirks.ResetMethod != QuirksResetUnset {
+			log.Debug(' ', "    init-reset = %s", quirks.ResetMethod)
+		}
 		for name, value := range quirks.HttpHeaders {
 			log.Debug(' ', "    http-%s = %q", strings.ToLower(name), value)
 		}
@@ -117,11 +123,17 @@ func NewUsbTransport(desc UsbDeviceDesc) (*UsbTransport, error) {
 	var maxconn uint
 
 	// Check for blacklisted device
-	if len(transport.quirks) > 0 && transport.quirks[0].Blacklist {
+	if transport.quirks.GetBlacklist() {
 		err = ErrBlackListed
 		goto ERROR
 	}
 
+	// Hard-reset the device, if needed
+	if transport.quirks.GetResetMethod() == QuirksResetHard {
+		transport.log.Debug(' ', "Doing USB HARD RESET")
+		dev.Reset()
+	}
+
 	// Configure the device
 	err = dev.Configure(desc)
 	if err != nil {
@@ -129,16 +141,14 @@ func NewUsbTransport(desc UsbDeviceDesc) (*UsbTransport, error) {
 	}
 
 	// Open connections
-	maxconn = math.MaxUint32
-	for _, quirks := range transport.quirks {
-		if quirks.UsbMaxInterfaces != 0 {
-			maxconn = quirks.UsbMaxInterfaces
-		}
+	maxconn = transport.quirks.GetUsbMaxInterfaces()
+	if maxconn == 0 {
+		maxconn = math.MaxUint32
 	}
 
 	for i, ifaddr := range desc.IfAddrs {
 		var conn *usbConn
-		conn, err = transport.openUsbConn(i, ifaddr)
+		conn, err = transport.openUsbConn(i, ifaddr, transport.quirks)
 		if err != nil {
 			goto ERROR
 		}
@@ -323,6 +333,11 @@ func (transport *UsbTransport) UsbDeviceInfo() UsbDeviceInfo {
 	return transport.info
 }
 
+// Quirks returns device's quirks
+func (transport *UsbTransport) Quirks() QuirksSet {
+	return transport.quirks
+}
+
 // RoundTrip implements http.RoundTripper interface
 func (transport *UsbTransport) RoundTrip(r *http.Request) (
 	*http.Response, error) {
@@ -423,6 +438,12 @@ func (transport *UsbTransport) RoundTripWithSession(session int,
 
 	transport.log.HTTPDebug(' ', session, "connection %d allocated", conn.index)
 
+	// Make an inter-request (or initial) delay, if needed
+	if delay := conn.delayUntil.Sub(time.Now()); delay > 0 {
+		transport.log.HTTPDebug(' ', session, "Pausing for %s", delay)
+		time.Sleep(delay)
+	}
+
 	// Send request and receive a response
 	err = outreq.Write(conn)
 	if err != nil {
@@ -545,17 +566,19 @@ func (wrap *usbResponseBodyWrapper) Close() error {
 
 // usbConn implements an USB connection
 type usbConn struct {
-	transport *UsbTransport // Transport that owns the connection
-	index     int           // Connection index (for logging)
-	iface     *UsbInterface // Underlying interface
-	reader    *bufio.Reader // For http.ReadResponse
-	cntRecv   int           // Total bytes received
-	cntSent   int           // Total bytes sent
+	transport     *UsbTransport // Transport that owns the connection
+	index         int           // Connection index (for logging)
+	iface         *UsbInterface // Underlying interface
+	reader        *bufio.Reader // For http.ReadResponse
+	delayUntil    time.Time     // Delay till this time before next request
+	delayInterval time.Duration // Pause between requests
+	cntRecv       int           // Total bytes received
+	cntSent       int           // Total bytes sent
 }
 
 // Open usbConn
 func (transport *UsbTransport) openUsbConn(
-	index int, ifaddr UsbIfAddr) (*usbConn, error) {
+	index int, ifaddr UsbIfAddr, quirks QuirksSet) (*usbConn, error) {
 
 	dev := transport.dev
 
@@ -563,8 +586,10 @@ func (transport *UsbTransport) openUsbConn(
 
 	// Initialize connection structure
 	conn := &usbConn{
-		transport: transport,
-		index:     index,
+		transport:     transport,
+		index:         index,
+		delayUntil:    time.Now().Add(quirks.GetInitDelay()),
+		delayInterval: quirks.GetRequestDelay(),
 	}
 
 	conn.reader = bufio.NewReader(conn)
@@ -576,18 +601,14 @@ func (transport *UsbTransport) openUsbConn(
 		goto ERROR
 	}
 
-	// Soft-reset interface
-	//
-	// Note, disabled for now, because it causes problems
-	// with EPSON ET-4750 (see #17)
-	//
-	// May be in a future we will enable it conditionally,
-	// for some printer models (based on quirks)
-	//
-	// err = conn.iface.SoftReset()
-	if err != nil {
-		// Don't treat it too seriously
-		transport.log.Info('?', "USB[%d]: SOFT_RESET: %s", index, err)
+	// Soft-reset interface, if needed
+	if quirks.GetResetMethod() == QuirksResetSoft {
+		transport.log.Debug(' ', "USB[%d]: doing SOFT_RESET", index)
+		err = conn.iface.SoftReset()
+		if err != nil {
+			// Don't treat it too seriously
+			transport.log.Info('?', "USB[%d]: SOFT_RESET: %s", index, err)
+		}
 	}
 
 	return conn, nil
@@ -648,6 +669,8 @@ func (conn *usbConn) Read(b []byte) (int, error) {
 			"USB[%d]: read: wanted %d got %d total %d",
 			conn.index, len(b), n, conn.cntRecv)
 
+		conn.transport.log.HexDump(LogTraceUSB, '<', b[:n])
+
 		if err != nil {
 			conn.transport.log.Error('!',
 				"USB[%d]: recv: %s", conn.index, err)
@@ -684,6 +707,8 @@ func (conn *usbConn) Write(b []byte) (int, error) {
 		"USB[%d]: write: wanted %d sent %d total %d",
 		conn.index, len(b), n, conn.cntSent)
 
+	conn.transport.log.HexDump(LogTraceUSB, '>', b[:n])
+
 	if err != nil {
 		conn.transport.log.Error('!',
 			"USB[%d]: send: %s", conn.index, err)
@@ -713,6 +738,7 @@ func (conn *usbConn) put() {
 	transport := conn.transport
 
 	conn.reader.Reset(conn)
+	conn.delayUntil = time.Now().Add(conn.delayInterval)
 	conn.cntRecv = 0
 	conn.cntSent = 0
 
